
#include "Integration_SHM_Fun.h"

int main(int argc, char *argv[])
{
  char InputARF[1000];
  char  OutputRate[1000];
  double MassDMGeV;
  
  if (argc>1) {strcpy(InputARF, argv[1]); }
  if (argc>2) {strcpy(OutputRate, argv[2]); }
  if (argc>3) { MassDMGeV=atof(argv[3]); }
  
  FILE *txtfile = fopen(InputARF,"r");
  FILE *outfile = fopen(OutputRate,"w");

  if (txtfile == NULL)
    {  
      printf("Error! ResFun File could not open file\n");
      exit(-1); 
    }
  
  if (outfile  == NULL)
    {  
      printf("Error! Outout File could not open file\n");
      exit(-1); 
    }

    // Constants
    const double PI = 3.14159265358979;
    const double percm_GeV = 1.97326971780039025e-14;
    const double d1 = 1E-9;
    const double rate_factor = pow(c1, 2) * 1E-18 * 1E+3 * 86400 * 3E+10 * 0.4 * (6.02214129e+23 * 1000.0 / 131.293) / MassDMGeV;
    double MassDM = MassDMGeV * 1E+9;  // eV

    double c = 299792458; //msec^{-1}
    
    double VESC = 544E+3/c;
    double V0 = 220E+3/c; 
    double VE = 232E+3/c;
    double Vmax = (VE + VESC);
    
    // Temporary variables
    double En, Mom, Res;

    std::vector<double> Momentum;
    std::vector<double> Respon;
    std::vector<double> Energy;

    while(!feof(txtfile))
      {
	fscanf(txtfile, "%lf  %lf %lf \n", &En, &Mom, &Res);
	Energy.push_back(En);
	Momentum.push_back(Mom);
	Respon.push_back(Res); 
      }
    
     // Final integration using trapezoidal rule
    std::vector<double> SingleMom, SingleResFun;   
    double dcs, Vmin;
    const int VelNum = 200; 
    double VelStepSize;
    std::vector<double> Vchi;
    std::vector<double> VspecRespInt;
    std::vector<double> dcsVel;
    std::vector<double> FinalMom;

    double tempVchi, tempInteg, tempVel;
    
    for (size_t aa = 0; aa < Energy.size() - 1; ++aa)
      {
        if (Energy[aa + 1] == Energy[aa])
	  {
	    SingleMom.push_back(Momentum[aa]);
	    SingleResFun.push_back(Respon[aa]);	    
	  }
	else
	  {
	    for(int gg = 0; gg < SingleMom.size();gg++)
	      {
		Vmin = Energy[aa]/SingleMom[gg] + SingleMom[gg]/(2*MassDM);
		
		if(Vmax > Vmin)
		  {
		    VelStepSize = (Vmax - Vmin)/VelNum;
		  
		    FinalMom.push_back(SingleMom[gg]);
		    
		    for(int ll = 0; ll<VelNum; ll++)
		      {
			tempVchi = Vmin + ll*VelStepSize;
			tempInteg = (VeloSpecInt(tempVchi, VE, VESC, V0, 500)/pow(tempVchi,2))*SingleResFun[gg]*(1.0/(2*PI*2.5682E-9))*(1.0/pow(SingleMom[gg],3));

			Vchi.push_back(tempVchi);
			VspecRespInt.push_back(tempInteg);
		      }
		    tempVel = trapezoidal_Integration(Vchi, VspecRespInt);
		    
		    dcsVel.push_back(tempVel);
		    Vchi.clear();
		    VspecRespInt.clear();		    
		  }
	      }
	    
	    

	    dcs = trapezoidal_Integration(FinalMom, dcsVel);
	    fprintf(outfile,"%lf  %e \n ", 1E-3*Energy[aa], rate_factor*dcs );

	    printf("%lf  %e \n ", 1E-3*Energy[aa], rate_factor*dcs );

	    dcsVel.clear();
	    FinalMom.clear();
	    SingleMom.clear();
	    SingleResFun.clear(); 
	  }

	
      }

 
    fclose(txtfile);
    fclose(outfile);
}
